SystemOrganization addCategory: #'AL-Http'!

Object subclass: #ALConnection
	instanceVariableNames: 'in out application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALConnection class methodsFor: 'as yet unclassified' stamp: 'cwp 4/24/2011 23:17'!
in: aReadStream out: aWriteStream application: anApplication
	^ self basicNew 
		initializeWithIn: aReadStream 
		out: aWriteStream 
		application: anApplication ! !

!ALConnection class methodsFor: 'instance creation' stamp: 'cwp 4/24/2011 23:17'!
socket: aSocket application: anApplication
	^ self in: aSocket reading out: aSocket writing application: anApplication! !

!ALConnection methodsFor: 'as yet unclassified' stamp: 'cwp 4/13/2011 21:54'!
close
	in close.
	out close.
	! !

!ALConnection methodsFor: 'running' stamp: 'cwp 8/20/2011 23:59'!
handleRequest
	| parser printer response |
	parser := ALHttpParser on: in.
	printer := ALHttpPrinter on: out.
	response := self handleRequest: parser parseRequest.
	printer printResponse: response.
	response writeEntityInto: out.
	response isConnectionClose ifTrue: [ self close ]! !

!ALConnection methodsFor: 'running' stamp: 'cwp 8/21/2011 00:01'!
handleRequest: aRequest
	^ [application handleRequest: aRequest]
		on: Error
		do: [:error | ALResponse status: 500 reason: error messageText]! !

!ALConnection methodsFor: 'running' stamp: 'cwp 4/13/2011 21:57'!
handleRequests
	[
		[[self handleRequest] repeat]
			on: Incomplete
			do: [:ex | self close]	
	] ifCurtailed: [self close]! !

!ALConnection methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 23:16'!
initializeWithIn: aReadStream out: aWriteStream application: anApplication
	self initialize.
	in := aReadStream.
	out := aWriteStream.
	application := anApplication ! !

Object subclass: #ALHttpParser
	instanceVariableNames: 'in ch request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALHttpParser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:51'!
on: aReadStream
	^ self basicNew initializeWithStream: aReadStream! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assert: aByte
	ch = aByte ifFalse: [ALMalformedRequest signal].
	ch := nil.! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertCrLf
	"Character cr asciiValue = 13"
	"Character lf acsiiValue = 10"
	
	self assert: 13.
	self next: 10.
! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertSpace
	"Character space asciiValue = 32"
	self assert: 32.
! !

!ALHttpParser methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:52'!
initializeWithStream: aReadStream
	in := aReadStream
	! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 20:09'!
next: aByte
	in get = aByte ifFalse: [ALMalformedRequest signal]! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/6/2011 10:34'!
parseFieldValue
	"Character tab asciiValue = 9"
	"Character space asciiValue = 32"
	"Character cr asciivalue = 13"
	"Character lf asciiValue = 10"
	"For now, we ignore the possibility of line folding"

	| out byte |
	[#(32 9) includes: (byte := in get)] whileTrue.
	out := ByteArray new writing.
	out put: byte.
	[(byte := in get) = 13] whileFalse: [out put: byte].
	self next: 10.
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 21:07'!
parseHeaders
	| name value |
	[name := self parseToken.
	name isEmpty ifTrue: [^ self assertCrLf].
	self assert: 58.
	value := self parseFieldValue.
	request header: name put: value]
		repeat
	! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:58'!
parseMethod
	| method |
	method := self parseToken.
	self assertSpace.
	^ method! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:00'!
parsePath
	"$/ asciiValue = 47"
	"Character space asciiValue = 32"

	| out path |
	path := ALPath root.
	ch := in get. 
	ch = 47 ifFalse: [self error].

	out := String new writing.
	[(ch := in get) = 32] whileFalse:
		[ch = 47
			ifFalse: [out put: (Character value: ch)]
			ifTrue: 
				[path := path / out conclusion.
				out := String new writing]].
	path := path / out conclusion.
	self assertSpace.
	^ path
			! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:13'!
parseRequest
	| method path version |
	method := self parseMethod.
	path := self parsePath.
	version := self parseVersion.
	request := ALRequest method: method path: path version: version.
	self parseHeaders.
	^ request! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:57'!
parseToken
	"These are the ascii values of characters that may be part of a token"

	|  legal out |
	legal :=  #(33 35 36 37 38 39 42 43 45 46 48 49 50 51 52 53 54 55 56 57 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 124 126).
	
	out := ByteArray new writing.
	ch ifNotNil: [out put: ch].
	[legal includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:11'!
parseVersion
	| version |
	self next: $H asciiValue.
	self next: $T asciiValue.
	self next: $T asciiValue.
	self next: $P asciiValue.
	self next: $/ asciiValue.
	version := self readAll: #[48 49 50 51 52 53 54 55 56 57 46].
	self assertCrLf.
	^ version! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:12'!
readAll: aByteArray
	| out |
	out := ByteArray new writing.
	[aByteArray includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

Object subclass: #ALHttpPrinter
	instanceVariableNames: 'out response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALHttpPrinter class methodsFor: 'as yet unclassified' stamp: 'cwp 4/23/2011 15:54'!
on: aWriteStream
	^ self basicNew initializeWithStream: aWriteStream ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:25'!
crlf
	"Character cr asciiValue -> 13"
	"Character lf asciiValue -> 10"

	out
		put: 13;
		put: 10! !

!ALHttpPrinter methodsFor: 'initialize-release' stamp: 'cwp 4/23/2011 15:55'!
initializeWithStream: aWriteStream
	out := aWriteStream  ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 5/4/2011 22:42'!
printHeader: name value: value
	out
		write: name;
		put: 58;
		put: 32;
		write: value.
	self crlf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 5/4/2011 22:43'!
printHeaders
	"$: asciiValue -> 58"

	"Character space asciiValue -> 32"

	response headersDo: [ :name :value | self printHeader: name value: value].
	self crlf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:26'!
printResponse
	self printStatusLine.
	self printHeaders.
! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:18'!
printResponse: aResponse
	response := aResponse.
	self printResponse.
	response := nil! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 4/23/2011 18:18'!
printStatusLine
	out
		write: 'HTTP/';
		write: (String streamContents: [ :s | response version printFractionAsDecimalOn: s ]);
		space;
		print: response status;
		space;
		write: response reason;
		cr;
		lf! !

Object subclass: #ALMimeMessage
	instanceVariableNames: 'headers body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

ALMimeMessage subclass: #ALEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALEntity class methodsFor: 'instance creation' stamp: 'cwp 4/23/2011 19:45'!
callback: aBlock
	^ self basicNew initializeWithBody: aBlock  ! !

!ALEntity class methodsFor: 'instance creation' stamp: 'cwp 4/23/2011 19:50'!
empty
	^ self callback: [:out]! !

!ALEntity class methodsFor: 'instance creation' stamp: 'cwp 4/23/2011 19:47'!
html: aString
	^ (self callback: [:out | (out encoding: #ascii) write: aString])
		header: 'Content-Type' put: 'text/html';
		header: 'Content-Length' put: aString length printString;
		yourself! !

!ALEntity methodsFor: 'setting' stamp: 'cwp 8/9/2011 23:57'!
contentLength: anInteger 
	self header: 'Content-Length' put: anInteger printString! !

!ALEntity methodsFor: 'setting' stamp: 'cwp 5/2/2011 18:31'!
contentType: aString 
	self header: 'Content-Type' put: aString! !

!ALEntity methodsFor: 'as yet unclassified' stamp: 'cwp 4/30/2011 23:35'!
writeEntityInto: aWriteStream
	body value: aWriteStream! !

ALMimeMessage subclass: #ALHttpMessage
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 5/4/2011 23:08'!
beConnectionKeepAlive
	self header: 'Connection' put: 'keep-alive'! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 5/4/2011 22:02'!
beTransferEncodingChunked
	self header: 'Transfer-Encoding' put: 'chunked'! !

!ALHttpMessage methodsFor: 'as yet unclassified' stamp: 'cwp 5/4/2011 22:43'!
headersDo: aBlock
	super headersDo: aBlock.
	body headersDo: aBlock.! !

!ALHttpMessage methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 21:17'!
initialize
	version := 1.1s1! !

!ALHttpMessage methodsFor: 'testing' stamp: 'cwp 5/4/2011 23:07'!
isConnectionClose
	^ (self header: 'Connection') == 'close'! !

!ALHttpMessage methodsFor: 'testing' stamp: 'cwp 4/24/2011 21:22'!
isConnectionKeepAlive
	^ (self header: 'Connection') = 'keep-alive'! !

!ALHttpMessage methodsFor: 'testing' stamp: 'cwp 5/2/2011 18:37'!
isTransferEncodingChunked
	^ (self header: 'Transfer-Encoding') = 'chunked'! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:16'!
version
	^ version! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:16'!
version: anObject
	version := anObject! !

ALHttpMessage subclass: #ALRequest
	instanceVariableNames: 'method path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 4/24/2011 21:00'!
method: aString path: aPath
	^ self basicNew
		initializeWithMethod: aString 
		path: aPath! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 4/24/2011 21:56'!
method: aString path: aPath version: aScaledDecimal 
	^ (self method: aString path: aPath) version: aScaledDecimal ! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 4/24/2011 20:58'!
method: aByteString path: aByteString2 version: aByteString3 body: anEntity
	^ self basicNew
		initializeWithMethod: aByteString
		path: aByteString2
		! !

!ALRequest methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 21:05'!
initializeWithMethod: aString path: aPath 
	self initializeWithBody: ALEntity empty.
	method := aString.
	path := aPath.
	! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 3/21/2011 22:25'!
isMethodGet
	^ method = 'GET'! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:01'!
method
	^ method! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 3/20/2011 22:58'!
path
	^ path! !

ALHttpMessage subclass: #ALResponse
	instanceVariableNames: 'status reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALResponse class methodsFor: 'constants' stamp: 'cwp 4/23/2011 19:27'!
defaultVersion
	^ 1.1s1! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 4/13/2011 23:01'!
notFound: aPath
	^ self status: 404 reason: 'Not found ', aPath printString! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 8/9/2011 16:00'!
seeOther: aPath
	^ (self status: 303 reason: 'See Other')
		header: 'Location' put: aPath printString;
		yourself! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 4/24/2011 22:04'!
status: anInteger reason: aString
	^ self basicNew
		initializeWithStatus: anInteger 
		reason: aString! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 5/2/2011 18:32'!
status: anInteger reason: aString body: anEntity
	^ (self status: anInteger reason: aString) 
		body: anEntity;
		yourself! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 4/23/2011 19:43'!
status: status reason: aString details: aString2
	| html |
	html := String new writing
		write: '<!!DOCTYPE html><html><head><title>';
		print: status;
		space;
		write: aString;
		write: '</title></head><body><h1>';
		write: aString;
		write: '</h1><p>';
		write: aString2;
		write: '</p></body></html>'.
	^ self status: 404 reason: 'Not Found' body: (ALEntity html: html)! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 5/2/2011 18:27'!
type: aString callback: aBlock
	| entity |
	entity := ALEntity callback: aBlock.
	entity contentType: aString.
	^ (self status: 200 reason: 'OK' body: entity)
		beTransferEncodingChunked;
		yourself! !

!ALResponse methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 21:09'!
initializeWithStatus: anInteger reason: aString
	self initializeWithBody: ALEntity empty.
	status := anInteger.
	reason := aString.
	! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
reason
	^ reason! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
status
	^ status! !

!ALResponse methodsFor: 'as yet unclassified' stamp: 'cwp 5/5/2011 00:33'!
writeEntityInto: aWriteStream
	self isTransferEncodingChunked
		ifFalse: [body writeEntityInto: aWriteStream]
		ifTrue: 
			[| stream |
			stream := (aWriteStream closing: []) chunking.
			body writeEntityInto: stream.
			stream close]! !

!ALMimeMessage methodsFor: 'as yet unclassified' stamp: 'cwp 5/2/2011 18:33'!
body: anObject
	 body := anObject! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 4/23/2011 19:19'!
header: aString 
	^ headers at: aString ifAbsent: [nil]! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 4/23/2011 19:17'!
header: aString put: anObject
	headers at: aString put: anObject! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 4/23/2011 19:17'!
headersDo: aBlock
	headers keysAndValuesDo: aBlock  ! !

!ALMimeMessage methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 21:04'!
initializeWithBody: anObject
	self initialize.
	headers := Dictionary new.
	body := anObject.! !

Object subclass: #ALServer
	instanceVariableNames: 'port process socket state application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:13'!
initialize
	Smalltalk addToStartUpList: self after: AutoStart.
	Smalltalk addToShutDownList: self! !

!ALServer class methodsFor: 'instance creation' stamp: 'cwp 4/23/2011 17:55'!
on: anInteger application: anApplication
	^ self basicNew initializeWithPort: anInteger application: anApplication! !

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:57'!
shutDown: aBoolean
	self allInstancesDo: [:ea | ea shutDown]! !

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:57'!
startUp: aBoolean
	self allInstancesDo: [:ea | ea startUp]! !

!ALServer methodsFor: 'initialize-release' stamp: 'cwp 5/4/2011 20:26'!
initializeWithPort: anInteger application: anApplication
	port := anInteger.
	application := anApplication.
	state := #stopped
	! !

!ALServer methodsFor: 'testing' stamp: 'cwp 4/13/2011 17:01'!
isRunning
	^ state == #running! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 20:23'!
loop
	[self serveConnection] repeat! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:16'!
printOn: aStream
	aStream nextPutAll: (self streamingPrintOn: String new writing)! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 20:27'!
run

	socket := Socket tcpCreateIfFail: [self error: 'Unable to create a socket'].
	socket
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: 1024;
		setOption: 'SO_RCVBUF' value: 1024;
		listenOn: port backlogSize: 10.
	socket isValid ifFalse: [self error: 'Unable to listen on port ', port printString].
	
	[self loop] ifCurtailed:   
		[socket destroy.
		socket := nil.
		process := nil].

! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 21:42'!
serveConnection
	| connection |
	connection := socket waitForAcceptFor: self timeout.
	connection ifNil: [^ self].
	[
		[(ALConnection socket: connection application: application) handleRequests] 
			forkAt: Processor highIOPriority 
			named: 'ALConnection to ', connection remoteAddress printString.
			
	] ifCurtailed: [connection destroy]! !

!ALServer methodsFor: 'running' stamp: 'cwp 8/20/2011 23:49'!
shutDown
	state == #running ifTrue:
		[process ifNotNil: [process terminate].
		state := #suspended]! !

!ALServer methodsFor: 'running' stamp: 'cwp 5/4/2011 20:35'!
start
	self isRunning ifTrue: [self stop].	
	state := #running.
	process := [self run] 
		forkAt: Processor lowIOPriority 
		named: self printString.
	! !

!ALServer methodsFor: 'running' stamp: 'cwp 5/4/2011 20:30'!
startUp
	state == #suspended ifTrue: [self start]! !

!ALServer methodsFor: 'running' stamp: 'cwp 8/6/2011 16:12'!
stop
	process ifNotNil: [process terminate].
	state := #stopped! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:18'!
streamingPrintOn: aWriteStream
	^ aWriteStream
		write: self class name;
		put: $[;
		print: port;
		put: $:;
		write: state;
		put: $];
		conclusion! !

!ALServer methodsFor: 'accessing' stamp: 'cwp 4/13/2011 17:07'!
timeout
	^ 5! !

Error subclass: #ALMalformedRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http'!
ALServer initialize!
